@@ -771,7 +771,7 @@ class TestS3SigV4(BaseS3OperationTest):
 
     def test_content_sha256_set_if_md5_is_unavailable(self):
         with mock.patch('botocore.auth.MD5_AVAILABLE', False):
-            with mock.patch('botocore.handlers.MD5_AVAILABLE', False):
+            with mock.patch('botocore.utils.MD5_AVAILABLE', False):
                 with self.http_stubber:
                     self.client.put_object(Bucket='foo', Key='bar', Body='baz')
         sent_headers = self.get_sent_headers()
@@ -1106,6 +1106,65 @@ class TestGeneratePresigned(BaseS3OperationTest):
             'get_object', {'Bucket': 'mybucket', 'Key': 'mykey'})
         self.assert_is_v2_presigned_url(url)
 
+def test_checksums_included_in_expected_operations():
+    """Validate expected calls include Content-MD5 header"""
+
+    t = S3ChecksumCases(_verify_checksum_in_headers)
+    yield t.case('put_bucket_tagging',
+            {"Bucket": "foo", "Tagging":{"TagSet":[]}})
+    yield t.case('put_bucket_lifecycle',
+            {"Bucket": "foo", "LifecycleConfiguration":{"Rules":[]}})
+    yield t.case('put_bucket_lifecycle_configuration',
+            {"Bucket": "foo", "LifecycleConfiguration":{"Rules":[]}})
+    yield t.case('put_bucket_cors',
+            {"Bucket": "foo", "CORSConfiguration":{"CORSRules": []}})
+    yield t.case('delete_objects',
+            {"Bucket": "foo", "Delete": {"Objects": [{"Key": "bar"}]}})
+    yield t.case('put_bucket_replication',
+            {"Bucket": "foo",
+             "ReplicationConfiguration": {"Role":"", "Rules": []}})
+    yield t.case('put_bucket_acl',
+            {"Bucket": "foo", "AccessControlPolicy":{}})
+    yield t.case('put_bucket_logging',
+            {"Bucket": "foo",
+             "BucketLoggingStatus":{}})
+    yield t.case('put_bucket_notification',
+            {"Bucket": "foo", "NotificationConfiguration":{}})
+    yield t.case('put_bucket_policy',
+            {"Bucket": "foo", "Policy": "<bucket-policy>"})
+    yield t.case('put_bucket_request_payment',
+            {"Bucket": "foo", "RequestPaymentConfiguration":{"Payer": ""}})
+    yield t.case('put_bucket_versioning',
+            {"Bucket": "foo", "VersioningConfiguration":{}})
+    yield t.case('put_bucket_website',
+            {"Bucket": "foo",
+             "WebsiteConfiguration":{}})
+    yield t.case('put_object_acl',
+            {"Bucket": "foo", "Key": "bar", "AccessControlPolicy":{}})
+    yield t.case('put_object_legal_hold',
+            {"Bucket": "foo", "Key": "bar", "LegalHold":{"Status": "ON"}})
+    yield t.case('put_object_retention',
+            {"Bucket": "foo", "Key": "bar",
+             "Retention":{"RetainUntilDate":"2020-11-05"}})
+    yield t.case('put_object_lock_configuration',
+            {"Bucket": "foo", "ObjectLockConfiguration":{}})
+
+
+def _verify_checksum_in_headers(operation, operation_kwargs):
+    environ = {}
+    with mock.patch('os.environ', environ):
+        environ['AWS_ACCESS_KEY_ID'] = 'access_key'
+        environ['AWS_SECRET_ACCESS_KEY'] = 'secret_key'
+        environ['AWS_CONFIG_FILE'] = 'no-exist-foo'
+        session = create_session()
+        session.config_filename = 'no-exist-foo'
+        client = session.create_client('s3')
+        with ClientHTTPStubber(client) as stub:
+            stub.add_response()
+            call = getattr(client, operation)
+            call(**operation_kwargs)
+            assert 'Content-MD5' in stub.requests[-1].headers
+
 
 def test_correct_url_used_for_s3():
     # Test that given various sets of config options and bucket names,
@@ -1249,6 +1308,10 @@ def test_correct_url_used_for_s3():
         s3_config=virtual_hosting,
         customer_provided_endpoint='https://foo.amazonaws.com',
         expected_url='https://bucket.foo.amazonaws.com/key')
+    yield t.case(
+        region='unknown', bucket='bucket', key='key',
+        s3_config=virtual_hosting,
+        expected_url='https://bucket.s3.unknown.amazonaws.com/key')
 
     # Test us-gov with virtual addressing.
     yield t.case(
@@ -1277,6 +1340,10 @@ def test_correct_url_used_for_s3():
         s3_config=path_style,
         customer_provided_endpoint='https://foo.amazonaws.com/',
         expected_url='https://foo.amazonaws.com/bucket/key')
+    yield t.case(
+        region='unknown', bucket='bucket', key='key',
+        s3_config=path_style,
+        expected_url='https://s3.unknown.amazonaws.com/bucket/key')
 
     # S3 accelerate
     use_accelerate = {'use_accelerate_endpoint': True}
@@ -1318,6 +1385,10 @@ def test_correct_url_used_for_s3():
         # Extra components must be whitelisted.
         customer_provided_endpoint='https://s3-accelerate.foo.amazonaws.com',
         expected_url='https://s3-accelerate.foo.amazonaws.com/bucket/key')
+    yield t.case(
+        region='unknown', bucket='bucket', key='key',
+        s3_config=use_accelerate,
+        expected_url='https://bucket.s3-accelerate.amazonaws.com/key')
     # Use virtual even if path is specified for s3 accelerate because
     # path style will not work with S3 accelerate.
     yield t.case(
@@ -1360,6 +1431,10 @@ def test_correct_url_used_for_s3():
         region='us-west-2', bucket='bucket', key='key',
         s3_config=use_dualstack, signature_version='s3v4',
         expected_url='https://bucket.s3.dualstack.us-west-2.amazonaws.com/key')
+    yield t.case(
+        region='unknown', bucket='bucket', key='key',
+        s3_config=use_dualstack, signature_version='s3v4',
+        expected_url='https://bucket.s3.dualstack.unknown.amazonaws.com/key')
     # Non DNS compatible buckets use path style for dual stack.
     yield t.case(
         region='us-west-2', bucket='bucket.dot', key='key',
@@ -1542,6 +1617,14 @@ def test_correct_url_used_for_s3():
             'unknown.amazonaws.com/key'
         )
     )
+    yield t.case(
+        region='unknown', bucket=accesspoint_arn, key='key',
+        s3_config={'use_arn_region': True},
+        expected_url=(
+            'https://myendpoint-123456789012.s3-accesspoint.'
+            'us-west-2.amazonaws.com/key'
+        )
+    )
     accesspoint_arn_cn = (
         'arn:aws-cn:s3:cn-north-1:123456789012:accesspoint:myendpoint'
     )
@@ -1681,6 +1764,11 @@ def test_correct_url_used_for_s3():
         s3_config=us_east_1_regional_endpoint,
         expected_url=(
             'https://bucket.s3.amazonaws.com/key'))
+    yield t.case(
+        region='unknown', bucket='bucket', key='key',
+        s3_config=us_east_1_regional_endpoint,
+        expected_url=(
+            'https://bucket.s3.unknown.amazonaws.com/key'))
     yield t.case(
         region='us-east-1', bucket='bucket', key='key',
         s3_config={
@@ -1723,11 +1811,22 @@ def test_correct_url_used_for_s3():
         expected_url=(
             'https://bucket.s3.amazonaws.com/key'))
 
+    yield t.case(
+        region='unknown', bucket='bucket', key='key',
+        s3_config=us_east_1_regional_endpoint_legacy,
+        expected_url=(
+            'https://bucket.s3.unknown.amazonaws.com/key'))
 
-class S3AddressingCases(object):
+
+class BaseTestCase:
     def __init__(self, verify_function):
         self._verify = verify_function
 
+    def case(self, **kwargs):
+        return self._verify, kwargs
+
+
+class S3AddressingCases(BaseTestCase):
     def case(self, region=None, bucket='bucket', key='key',
              s3_config=None, is_secure=True, customer_provided_endpoint=None,
              expected_url=None, signature_version=None):
@@ -1737,6 +1836,11 @@ class S3AddressingCases(object):
         )
 
 
+class S3ChecksumCases(BaseTestCase):
+    def case(self, operation, operation_args):
+        return self._verify, operation, operation_args
+
+
 def _verify_expected_endpoint_url(region, bucket, key, s3_config,
                                   is_secure=True,
                                   customer_provided_endpoint=None,
